"use strict";
/**
 * That file contain a list of function that can be use as API instead of
 * using the globular object itself.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.readLogs = exports.readErrors = exports.getReferencedValue = exports.getServiceBundles = exports.FindPackages = exports.setServicesDescriptor = exports.GetPackagesDescriptor = exports.getPackageDescriptor = exports.saveApplication = exports.deleteApplication = exports.removeActionFromApplication = exports.appendActionToApplication = exports.getAllApplicationsInfo = exports.deleteGroup = exports.createGroup = exports.removeMemberFromGroup = exports.appendMemberToGroup = exports.getAllGroups = exports.deleteRole = exports.createRole = exports.removeAccountRole = exports.addAccountRole = exports.removeActionFromRole = exports.appendActionToRole = exports.getAllRoles = exports.readUserData = exports.readOneUserData = exports.appendUserData = exports.refreshToken = exports.authenticate = exports.appendRoleToAccount = exports.removeRoleFromAccount = exports.deleteAccount = exports.registerAccount = exports.getAllAccountsInfo = exports.queryTsRange = exports.queryTs = exports.createDir = exports.readDir = exports.downloadDir = exports.downloadFileHttp = exports.createArchive = exports.deleteDir = exports.deleteFile = exports.renameFile = exports.uploadFiles = exports.OwnerType = exports.setActionPermission = exports.hasRuningProcess = exports.getErrorMessage = void 0;
exports.deletePeer = exports.removeActionFromPeer = exports.appendActionToPeer = exports.getAllPeersInfo = exports.createPeer = exports.searchDocuments = exports.pingSql = exports.getNumbeOfLogsByMethod = exports.deleteLogEntry = exports.clearAllLog = void 0;
var admin_pb_1 = require("./admin/admin_pb");
var monitoring_pb_1 = require("./monitoring/monitoring_pb");
var resource_pb_1 = require("./resource/resource_pb");
var log_pb_1 = require("./log/log_pb");
var jwt = require("jwt-decode");
var persistence_pb_1 = require("./persistence/persistence_pb");
var file_pb_1 = require("./file/file_pb");
var search_pb_1 = require("./search/search_pb");
var authentication_pb_1 = require("./authentication/authentication_pb");
// Here I will get the authentication information.
var domain = window.location.hostname;
var address = domain + ":" + window.location.port;
var application = window.location.pathname.split("/").join("");
function mergeTypedArrays(a, b) {
    // Checks for truthy values on both arrays
    if (!a && !b)
        throw 'Please specify valid arguments for parameters a and b.';
    // Checks for truthy values or empty arrays on each argument
    // to avoid the unnecessary construction of a new array and
    // the type comparison
    if (!b || b.length === 0)
        return a;
    if (!a || a.length === 0)
        return b;
    // Make sure that both typed arrays are of the same type
    if (Object.prototype.toString.call(a) !== Object.prototype.toString.call(b))
        throw 'The types of the two arguments passed for parameters a and b do not match.';
    var c = new a.constructor(a.length + b.length);
    c.set(a);
    c.set(b, a.length);
    return c;
}
function uint8arrayToStringMethod(myUint8Arr) {
    return String.fromCharCode.apply(null, myUint8Arr);
}
function getToken() {
    var token = localStorage.getItem("user_token");
    if (token == null) {
        return "";
    }
    return token;
}
/**
 * Display the error message to the end user.
 * @param err The error message can be a simple string or a json stringnify object
 */
function getErrorMessage(err) {
    try {
        var errObj = JSON.parse(err);
        if (errObj.ErrorMsg != undefined) {
            return errObj.ErrorMsg;
        }
    }
    catch (_a) {
        return err;
    }
}
exports.getErrorMessage = getErrorMessage;
/**
 *
 * @param globular
 * @param name
 * @param callback
 */
function hasRuningProcess(globular, name, callback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new admin_pb_1.HasRunningProcessRequest();
    rqst.setName(name);
    globular.adminService
        .hasRunningProcess(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (err) {
        callback(false);
    });
}
exports.hasRuningProcess = hasRuningProcess;
/**
 * Test if a process with a given name is running on the server.
 * @param globular
 * @param name
 * @param callback
 */
function hasRunningProcess(globular, name, callback) {
    var rqst;
    rqst.setName(name);
    globular.adminService
        .hasRunningProcess(rqst)
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (err) {
        callback(false);
    });
}
///////////////////////////////////// Resource & Permissions operations /////////////////////////////////
/**
 * Set/create action permission.
 * @param globular
 * @param application
 * @param domain
 * @param action
 * @param permission
 * @param callback
 * @param errorCallback
 */
function setActionPermission(globular, action, permission, callback, errorCallback) {
    /*
      const rqst = new SetActionPermissionRqst
      rqst.setAction(action)
      rqst.setPermission(permission)
  
      // Call set action permission.
      globular.resourceService.setActionPermission(rqst, {
          "token": token,
          "application": application, "domain": domain
      }).then(callback)
          .catch((err: any) => {
              errorCallback(err)
          })*/
}
exports.setActionPermission = setActionPermission;
/**
 * The permission can be assigned to
 * a User, a Role or an Application.
 */
var OwnerType;
(function (OwnerType) {
    OwnerType[OwnerType["User"] = 1] = "User";
    OwnerType[OwnerType["Role"] = 2] = "Role";
    OwnerType[OwnerType["Application"] = 3] = "Application";
})(OwnerType = exports.OwnerType || (exports.OwnerType = {}));
///////////////////////////////////// File operations /////////////////////////////////
function uploadFiles(globule, token, path, files, completeHandler, errorHandler, progressHandler, abortHandler, port, abortFct) {
    var fd = new FormData();
    // add all selected files
    for (var i = 0; i < files.length; i++) {
        var file = files[i];
        fd.append("multiplefiles", file, file.name);
        fd.append("path", path);
    }
    // create the request
    var xhr = new XMLHttpRequest();
    // Connect handling functions.
    xhr.upload.addEventListener("progress", progressHandler, false);
    xhr.addEventListener("error", errorHandler, false);
    xhr.addEventListener("abort", abortHandler, false);
    // The load event...
    xhr.onload = function () {
        if (xhr.status >= 200 && xhr.status < 300) {
            // we done! I will use the rename file event to refresh the directory...
            if (completeHandler != null) {
                completeHandler();
            }
        }
        else {
            // display error message...
            errorHandler(xhr.responseText);
            return;
        }
    };
    // path to server would be where you'd normally post the form to  
    var url = globule.config.Protocol + "://" + globule.config.Name;
    if (globule.config.Domain != "localhost" && globule.config.Domain.length > 0) {
        url += "." + globule.config.Domain;
    }
    if (globule.config.Protocol == "https") {
        if (globule.config.PortHttps != 443)
            url += ":" + globule.config.PortHttps;
    }
    else {
        if (globule.config.PortHttps != 80)
            url += ":" + globule.config.PortHttp;
    }
    // Set the values also as parameters...
    url += "/uploads";
    url += "?domain=" + globule.domain;
    url += "&application=" + application;
    if (localStorage.getItem("user_token") != undefined) {
        url += "&token=" + localStorage.getItem("user_token");
    }
    xhr.open("POST", url, true);
    // path to server would be where you'd normally post the form to
    xhr.setRequestHeader("token", token);
    xhr.setRequestHeader("application", application);
    xhr.setRequestHeader("domain", globule.domain);
    xhr.send(fd);
    return xhr;
}
exports.uploadFiles = uploadFiles;
/**
 * Rename a file or a directorie with given name.
 * @param path The path inside webroot
 * @param newName The new file name
 * @param oldName  The old file name
 * @param callback  The success callback.
 * @param errorCallback The error callback.
 */
function renameFile(globular, path, newName, oldName, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new file_pb_1.RenameRequest();
    // path = path.replace("/webroot", ""); // remove the /webroot part.
    if (path.length === 0) {
        path = "/";
    }
    path = encodeURI(path);
    rqst.setPath(path);
    rqst.setOldName(oldName);
    rqst.setNewName(newName);
    globular.fileService
        .rename(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
        path: path + "/" + oldName,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (error) {
        if (errorCallback !== undefined) {
            errorCallback(error);
        }
    });
}
exports.renameFile = renameFile;
/**
 * Delete a file with a given path.
 * @param path The path of the file to be deleted.
 * @param callback The success callback.
 * @param errorCallback The error callback.
 */
function deleteFile(globular, path, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new file_pb_1.DeleteFileRequest();
    path = path.replace(globular.config.WebRoot, ""); // remove the /webroot part.
    if (path.length === 0) {
        path = "/";
    }
    path = encodeURI(path);
    rqst.setPath(path);
    globular.fileService
        .deleteFile(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
        path: path,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (error) {
        if (errorCallback !== undefined) {
            errorCallback(error);
        }
    });
}
exports.deleteFile = deleteFile;
/**
 * Remove a given directory and all element it contain.
 * @param path The path of the directory to be deleted.
 * @param callback The success callback
 * @param errorCallback The error callback.
 */
function deleteDir(globular, path, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new file_pb_1.DeleteDirRequest();
    // path = path.replace("/webroot", ""); // remove the /webroot part.
    if (path.length === 0) {
        path = "/";
    }
    path = encodeURI(path);
    rqst.setPath(path);
    globular.fileService
        .deleteDir(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
        path: path,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (error) {
        if (errorCallback !== undefined) {
            errorCallback(error);
        }
    });
}
exports.deleteDir = deleteDir;
/**
 * Create a dir archive.
 * @param path
 * @param name
 * @param callback
 * @param errorCallback
 */
function createArchive(globular, paths, name, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new file_pb_1.CreateArchiveRequest();
    rqst.setPathsList(paths);
    rqst.setName(name);
    globular.fileService
        .createArchive(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (error) {
        if (errorCallback !== undefined) {
            errorCallback(error);
        }
    });
}
exports.createArchive = createArchive;
/**
 * Download a file from the server.
 * @param urlToSend
 */
function downloadFileHttp(url, fileName, callback, token) {
    if (token === void 0) { token = getToken(); }
    var req = new XMLHttpRequest();
    req.open("GET", url, true);
    // Set the token to manage downlaod access.
    req.setRequestHeader("token", token);
    req.setRequestHeader("application", application);
    req.setRequestHeader("domain", domain);
    req.responseType = "blob";
    req.onload = function (event) {
        var blob = req.response;
        var link = document.createElement("a");
        link.href = window.URL.createObjectURL(blob);
        link.download = fileName;
        link.click();
        callback();
    };
    req.send();
}
exports.downloadFileHttp = downloadFileHttp;
/**
 * Download a directory as archive file. (.tar.gz)
 * @param path The path of the directory to dowload.
 * @param callback The success callback.
 * @param errorCallback The error callback.
 */
function downloadDir(globular, path, callback, errorCallback, port, token) {
    if (token === void 0) { token = getToken(); }
    var name = path.split("/")[path.split("/").length - 1];
    // path = path.replace("/webroot", ""); // remove the /webroot part.
    // append the port to the url.
    if (port != undefined) {
        path += ":" + port + path;
    }
    // Create an archive-> download it-> delete it...
    createArchive(globular, [path], name, function (_path) {
        // display the archive path...
        var url = window.location.protocol + "//" + window.location.hostname;
        if (port != undefined) {
            url += ":" + port;
        }
        url += _path;
        downloadFileHttp(url, name, function () {
            // Here the file was downloaded I will now delete it.
            setTimeout(function () {
                // wait a little and remove the archive from the server.
                var rqst = new file_pb_1.DeleteFileRequest();
                rqst.setPath(encodeURI(path + "/" + name));
                globular.fileService
                    .deleteFile(rqst, {
                    token: token,
                    application: application.length > 0 ? application : globular.config.IndexApplication,
                    domain: domain, address: address,
                    path: path,
                })
                    .then(callback)
                    .catch(errorCallback);
            }, 5000); // wait 5 second, arbritary...
        });
    }, errorCallback);
}
exports.downloadDir = downloadDir;
// Merge tow array together.
function mergeTypedArraysUnsafe(a, b) {
    var c = new a.constructor(a.length + b.length);
    c.set(a);
    c.set(b, a.length);
    return c;
}
/**
 * Read the content of a dir from a given path.
 * @param path The parent path of the dir to be read.
 * @param callback  Return the path of the dir with more information.
 * @param errorCallback Return a error if the file those not contain the value.
 */
function readDir(globular, path, recursive, callback, errorCallback, onLoaded, thumbnail_height, thumbnail_width, token) {
    if (thumbnail_height === void 0) { thumbnail_height = 80; }
    if (thumbnail_width === void 0) { thumbnail_width = 80; }
    if (token === void 0) { token = getToken(); }
    // path = path.replace("/webroot", ""); // remove the /webroot part.
    if (path.length === 0) {
        path = "/";
    }
    path = encodeURI(path);
    // So here I will encode the path to be sure no emoji string or other weird character will
    // make the request fail..
    var rqst = new file_pb_1.ReadDirRequest();
    rqst.setPath(path);
    rqst.setRecursive(recursive);
    rqst.setThumbnailheight(thumbnail_height);
    rqst.setThumbnailwidth(thumbnail_width);
    var stream = globular.fileService.readDir(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
        path: path,
    });
    var files;
    files = {};
    var dir;
    stream.on("data", function (rsp) {
        var f = rsp.getInfo();
        var filePath = f.getPath().split("//").join("/");
        var parent = files[filePath.substring(0, filePath.lastIndexOf("/"))];
        if (parent) {
            var files_ = parent.getFilesList();
            if (files[f.getPath()] == undefined) {
                files_.push(f);
                parent.setFilesList(files_);
            }
            // Here I will generate a local event... can be use to 
            // make the interface more responsive...
            globular.eventHub.publish("_read_dir_" + rqst.getPath(), f, true);
        }
        files[f.getPath()] = f;
        files[filePath] = f;
        if (dir == undefined && f.getIsDir()) {
            dir = f;
            files[path] = f;
        }
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            // return the dir initialyse with all it files infos...
            var path_1 = decodeURIComponent(rqst.getPath());
            callback(files[path_1]);
        }
        else {
            // error here...
            errorCallback({ message: status.details });
        }
    });
    /** that function is call when the file is read completetly... */
    stream.on("end", function () {
        if (onLoaded != undefined) {
            onLoaded(dir);
        }
    });
}
exports.readDir = readDir;
/**
 * Test if a file is contain in a list of files.
 * @param files
 */
function fileExist(fileName, files) {
    if (files != null) {
        for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
            var file = files_1[_i];
            if (file.Name === fileName) {
                return true;
            }
        }
    }
    return false;
}
/**
 * Create a new directory inside existing one.
 * @param path The path of the directory
 * @param callback The callback
 * @param errorCallback The error callback
 */
function createDir(globular, path, name, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    // path = path.replace("/webroot", ""); // remove the /webroot part.
    if (path.length === 0) {
        path = "/";
    }
    path = encodeURI(path);
    // Set the request.
    var rqst = new file_pb_1.CreateDirRequest();
    rqst.setPath(path);
    rqst.setName(name);
    // Create a directory at the given path.
    globular.fileService
        .createDir(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
        path: path,
    })
        .then(function () {
        // The new directory was created.
        callback(name);
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.createDir = createDir;
///////////////////////////////////// Time series Query //////////////////////////////////////
/**
 * Run a query over a time series database.
 * @param globular
 * @param application
 * @param domain
 * @param connectionId
 * @param query
 * @param ts
 * @param callback
 * @param errorCallback
 */
function queryTs(globular, connectionId, query, ts, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    // Create a new request.
    var rqst = new monitoring_pb_1.QueryRequest();
    rqst.setConnectionid(connectionId);
    rqst.setQuery(query);
    rqst.setTs(ts);
    // Now I will test with promise
    globular.monitoringService
        .query(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (resp) {
        if (callback !== undefined) {
            callback(JSON.parse(resp.getValue()));
        }
    })
        .catch(function (error) {
        if (errorCallback !== undefined) {
            errorCallback(error);
        }
    });
}
exports.queryTs = queryTs;
/**
 * Run query over a time series
 * @param globular
 * @param application
 * @param domain
 * @param connectionId
 * @param query
 * @param startTime
 * @param endTime
 * @param step
 * @param callback
 * @param errorCallback
 */
function queryTsRange(globular, connectionId, query, startTime, endTime, step, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    // Create a new request.
    var rqst = new monitoring_pb_1.QueryRangeRequest();
    rqst.setConnectionid(connectionId);
    rqst.setQuery(query);
    rqst.setStarttime(startTime);
    rqst.setEndtime(endTime);
    rqst.setStep(step);
    var buffer = { value: "", warning: "" };
    var stream = globular.monitoringService.queryRange(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    stream.on("data", function (rsp) {
        buffer.value += rsp.getValue();
        buffer.warning = rsp.getWarnings();
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            callback(JSON.parse(buffer.value));
        }
        else {
            errorCallback({ message: status.details });
        }
    });
    stream.on("end", function () {
        // stream end signal
    });
}
exports.queryTsRange = queryTsRange;
///////////////////////////////////// Account management action //////////////////////////////////////
/**
 * Return the list of all account on the server, guest and admin are new account...
 * @param globular
 * @param application
 * @param domain
 * @param callback
 * @param errorCallback
 */
function getAllAccountsInfo(globular, callback, errorCallback) {
    var rqst = new persistence_pb_1.FindRqst();
    rqst.setCollection("Accounts");
    rqst.setDatabase("local_resource");
    rqst.setId("local_resource");
    rqst.setQuery(""); // means all values.
    var stream = globular.persistenceService.find(rqst, {
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var data = [];
    stream.on("data", function (rsp) {
        data = mergeTypedArrays(data, rsp.getData());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            var accounts = JSON.parse(uint8arrayToStringMethod(data));
            callback(accounts);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.getAllAccountsInfo = getAllAccountsInfo;
/**
 * Register a new account.
 * @param userName The name of the account
 * @param email The email
 * @param password The password
 * @param confirmPassword
 * @param callback
 * @param errorCallback
 */
function registerAccount(globular, userName, email, password, confirmPassword, callback, errorCallback) {
    var request = new resource_pb_1.RegisterAccountRqst();
    var account = new resource_pb_1.Account();
    account.setPassword(password);
    account.setName(userName);
    account.setEmail(email);
    request.setAccount(account);
    request.setConfirmPassword(confirmPassword);
    // Create the user account.
    globular.resourceService
        .registerAccount(request, {
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.registerAccount = registerAccount;
/**
 * Remove an account from the server.
 * @param name  The _id of the account.
 * @param callback The callback when the action succeed
 * @param errorCallback The error callback.
 */
function deleteAccount(globular, id, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.DeleteAccountRqst();
    rqst.setId(id);
    // Remove the account from the database.
    globular.resourceService
        .deleteAccount(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.deleteAccount = deleteAccount;
/**
 * Remove a role from an account.
 * @param accountId The account id
 * @param roleId The role name (id)
 * @param callback The success callback
 * @param errorCallback The error callback
 */
function removeRoleFromAccount(globular, accountId, roleId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.RemoveAccountRoleRqst();
    rqst.setAccountid(accountId);
    rqst.setRoleid(roleId);
    globular.resourceService
        .removeAccountRole(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.removeRoleFromAccount = removeRoleFromAccount;
/**
 * Append a role to an account.
 * @param accountId The account id
 * @param roleId The role name (id)
 * @param callback The success callback
 * @param errorCallback The error callback.
 */
function appendRoleToAccount(globular, accountId, roleId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.AddAccountRoleRqst();
    rqst.setAccountid(accountId);
    rqst.setRoleid(roleId);
    globular.resourceService
        .addAccountRole(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.appendRoleToAccount = appendRoleToAccount;
/**
 * Authenticate the user and get the token
 * @param globular
 * @param eventHub
 * @param application
 * @param domain
 * @param userName
 * @param password
 * @param callback
 * @param errorCallback
 */
function authenticate(globular, eventHub, userName, password, callback, errorCallback) {
    var rqst = new authentication_pb_1.AuthenticateRqst();
    rqst.setName(userName);
    rqst.setPassword(password);
    // Create the user account.
    globular.authenticationService
        .authenticate(rqst, { application: application.length > 0 ? application : globular.config.IndexApplication, domain: domain, address: address, })
        .then(function (rsp) {
        // Here I will set the token in the localstorage.
        var token = rsp.getToken();
        var decoded = jwt(token);
        // here I will save the user token and user_name in the local storage.
        localStorage.setItem("user_token", token);
        localStorage.setItem("user_name", decoded.username);
        // Publish local login event.
        eventHub.publish("onlogin", globular.config, true); // return the full config...
        callback(decoded);
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.authenticate = authenticate;
/**
 * Function to be use to refresh token.
 * @param globular
 * @param eventHub
 * @param application
 * @param domain
 * @param callback
 * @param errorCallback
 */
function refreshToken(globular, eventHub, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new authentication_pb_1.RefreshTokenRqst();
    rqst.setToken(localStorage.getItem("user_token"));
    globular.authenticationService
        .refreshToken(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        // Here I will set the token in the localstorage.
        var token = rsp.getToken();
        var decoded = jwt(token);
        // here I will save the user token and user_name in the local storage.
        localStorage.setItem("user_token", token);
        localStorage.setItem("user_name", decoded.username);
        // Publish local login event.
        eventHub.publish("onlogin", globular.config, true); // return the full config...
        callback(decoded);
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.refreshToken = refreshToken;
/**
 * Save user data into the user_data collection.
 * @param globular
 * @param application
 * @param domain
 * @param data
 * @param callback
 * @param errorCallback
 */
function appendUserData(globular, data, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var userName = localStorage.getItem("user_name");
    var database = userName + "_db";
    var collection = "user_data";
    var rqst = new persistence_pb_1.InsertOneRqst();
    rqst.setId(database);
    rqst.setDatabase(database);
    rqst.setCollection(collection);
    rqst.setData(JSON.stringify(data));
    rqst.setOptions("");
    // call persist data
    globular.persistenceService
        .insertOne(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getId());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.appendUserData = appendUserData;
/**
 * Read user data one result at time.
 * @param globular
 * @param application
 * @param domain
 * @param query
 * @param callback
 * @param errorCallback
 */
function readOneUserData(globular, query, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var userName = localStorage.getItem("user_name");
    var database = userName + "_db";
    var collection = "user_data";
    var rqst = new persistence_pb_1.FindOneRqst();
    rqst.setId(database);
    rqst.setDatabase(database);
    rqst.setCollection(collection);
    rqst.setQuery(query);
    rqst.setOptions("");
    // call persist data
    globular.persistenceService
        .findOne(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(JSON.parse(rsp.getJsonstr()));
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.readOneUserData = readOneUserData;
/**
 * Read all user data.
 * @param globular
 * @param application
 * @param domain
 * @param query
 * @param callback
 * @param errorCallback
 */
function readUserData(globular, query, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var userName = localStorage.getItem("user_name");
    var database = userName + "_db";
    var collection = "user_data";
    var rqst = new persistence_pb_1.FindOneRqst();
    rqst.setId(database);
    rqst.setDatabase(database);
    rqst.setCollection(collection);
    rqst.setQuery(query);
    rqst.setOptions("");
    // call persist data
    var stream = globular.persistenceService.find(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var data = [];
    // Get the stream and set event on it...
    stream.on("data", function (rsp) {
        data = mergeTypedArrays(data, rsp.getData());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            var results = JSON.parse(uint8arrayToStringMethod(data));
            callback(results);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.readUserData = readUserData;
///////////////////////////////////// Role action //////////////////////////////////////
/**
 * Retreive the list of all available roles on the server.
 * @param callback That function is call in case of success.
 * @param errorCallback That function is call in case error.
 */
function getAllRoles(globular, callback, errorCallback) {
    var rqst = new resource_pb_1.GetRolesRqst;
    rqst.setQuery(""); // means all values.
    var stream = globular.resourceService.getRoles(rqst, {
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var data = new Array();
    stream.on("data", function (rsp) {
        data = data.concat(rsp.getRolesList());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            callback(data);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.getAllRoles = getAllRoles;
/**
 * Append Action to a given role.
 * @param action The action name.
 * @param role The role.
 * @param callback The success callback
 * @param errorCallback The error callback.
 */
function appendActionToRole(globular, role, actions, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.AddRoleActionsRqst();
    rqst.setRoleid(role);
    rqst.setActionsList(actions);
    globular.resourceService
        .addRoleActions(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.appendActionToRole = appendActionToRole;
/**
 * Remove the action from a given role.
 * @param action The action id
 * @param role The role id
 * @param callback success callback
 * @param errorCallback error callback
 */
function removeActionFromRole(globular, role, action, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.RemoveRoleActionRqst();
    rqst.setRoleid(role);
    rqst.setAction(action);
    globular.resourceService
        .removeRoleAction(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.removeActionFromRole = removeActionFromRole;
/**
 * Append member to a given group.
 * @param roleId the role id
 * @param accountId The account id.
 * @param callback The success callback
 * @param errorCallback The error callback.
 */
function addAccountRole(globular, roleId, accountId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.AddAccountRoleRqst;
    rqst.setRoleid(roleId);
    rqst.setAccountid(accountId);
    globular.resourceService
        .addAccountRole(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.addAccountRole = addAccountRole;
/**
 * Remove the action from a given role.
 * @param action The action id
 * @param role The role id
 * @param callback success callback
 * @param errorCallback error callback
 */
function removeAccountRole(globular, roleId, accountId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.RemoveAccountRoleRqst;
    rqst.setRoleid(roleId);
    rqst.setAccountid(accountId);
    globular.resourceService
        .removeAccountRole(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.removeAccountRole = removeAccountRole;
/**
 * Create a new Role
 * @param globular
 * @param application
 * @param domain
 * @param id
 * @param callback
 * @param errorCallback
 */
function createRole(globular, id, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.CreateRoleRqst();
    var role = new resource_pb_1.Role();
    role.setId(id);
    role.setName(id);
    rqst.setRole(role);
    globular.resourceService
        .createRole(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.createRole = createRole;
/**
 * Delete a given role
 * @param globular
 * @param application
 * @param domain
 * @param id
 * @param callback
 * @param errorCallback
 */
function deleteRole(globular, id, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.DeleteRoleRqst();
    rqst.setRoleid(id);
    globular.resourceService
        .deleteRole(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.deleteRole = deleteRole;
///////////////////////////////////// Group Operation ////////////////////////////////////////
/**
 * Retreive the list of all available group on the server.
 * @param callback That function is call in case of success.
 * @param errorCallback That function is call in case error.
 */
function getAllGroups(globular, callback, errorCallback) {
    var rqst = new resource_pb_1.GetGroupsRqst();
    rqst.setQuery(""); // means all values.
    var stream = globular.resourceService.getGroups(rqst, {
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var data = new Array();
    stream.on("data", function (rsp) {
        data = data.concat(rsp.getGroupsList());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            callback(data);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.getAllGroups = getAllGroups;
/**
 * Append member to a given group.
 * @param groupId to group
 * @param accountId The account.
 * @param callback The success callback
 * @param errorCallback The error callback.
 */
function appendMemberToGroup(globular, groupId, accountId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.AddGroupMemberAccountRqst;
    rqst.setGroupid(groupId);
    rqst.setAccountid(accountId);
    globular.resourceService
        .addGroupMemberAccount(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.appendMemberToGroup = appendMemberToGroup;
/**
 * Remove the action from a given role.
 * @param action The action id
 * @param role The role id
 * @param callback success callback
 * @param errorCallback error callback
 */
function removeMemberFromGroup(globular, groupId, accountId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.RemoveGroupMemberAccountRqst;
    rqst.setGroupid(groupId);
    rqst.setAccountid(accountId);
    globular.resourceService
        .removeGroupMemberAccount(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.removeMemberFromGroup = removeMemberFromGroup;
/**
 * Create a new Group
 * @param globular
 * @param application
 * @param domain
 * @param id
 * @param callback
 * @param errorCallback
 */
function createGroup(globular, id, name, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.CreateGroupRqst();
    var group = new resource_pb_1.Group();
    group.setId(id);
    group.setName(id);
    rqst.setGroup(group);
    globular.resourceService
        .createGroup(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.createGroup = createGroup;
/**
 * Delete a given role
 * @param globular
 * @param application
 * @param domain
 * @param id
 * @param callback
 * @param errorCallback
 */
function deleteGroup(globular, id, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.DeleteGroupRqst();
    rqst.setGroup(id);
    globular.resourceService
        .deleteGroup(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.deleteGroup = deleteGroup;
///////////////////////////////////// Application operations /////////////////////////////////
/**
 * Return the list of all application
 * @param globular
 * @param application
 * @param domain
 * @param callback
 * @param errorCallback
 */
function getAllApplicationsInfo(globular, callback, errorCallback) {
    var rqst = new resource_pb_1.GetApplicationsRqst();
    var stream = globular.resourceService.getApplications(rqst, {
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var applications = new Array();
    stream.on("data", function (rsp) {
        applications = applications.concat(rsp.getApplicationsList());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            callback(applications);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.getAllApplicationsInfo = getAllApplicationsInfo;
/**
 * Append action to application.
 * @param globular
 * @param application
 * @param domain
 * @param applicationId
 * @param action
 * @param callback
 * @param errorCallback
 */
function appendActionToApplication(globular, applicationId, actions, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.AddApplicationActionsRqst();
    rqst.setApplicationid(applicationId);
    rqst.setActionsList(actions);
    globular.resourceService
        .addApplicationActions(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.appendActionToApplication = appendActionToApplication;
/**
 * Remove action from application.
 * @param globular
 * @param application
 * @param domain
 * @param action
 * @param callback
 * @param errorCallback
 */
function removeActionFromApplication(globular, action, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.RemoveApplicationActionRqst();
    rqst.setApplicationid(application);
    rqst.setAction(action);
    globular.resourceService
        .removeApplicationAction(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.removeActionFromApplication = removeActionFromApplication;
/**
 * Delete application
 * @param globular
 * @param application
 * @param domain
 * @param applicationId
 * @param callback
 * @param errorCallback
 */
function deleteApplication(globular, applicationId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.DeleteApplicationRqst();
    rqst.setApplicationid(applicationId);
    globular.resourceService
        .deleteApplication(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.deleteApplication = deleteApplication;
/**
 * Save application
 * @param globular
 * @param eventHub
 * @param applicationId
 * @param domain
 * @param application
 * @param callback
 * @param errorCallback
 */
function saveApplication(globular, eventHub, _application, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new persistence_pb_1.ReplaceOneRqst();
    rqst.setCollection("Applications");
    rqst.setDatabase("local_resource");
    rqst.setId("local_resource");
    rqst.setValue(JSON.stringify(_application));
    rqst.setQuery("{\"_id\":\"".concat(_application._id, "\"}")); // means all values.
    globular.persistenceService
        .replaceOne(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        eventHub.publish("update_application_info_event", JSON.stringify(application), false);
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.saveApplication = saveApplication;
///////////////////////////////////// Services operations /////////////////////////////////
/**
 * Return a list of service descriptor related to a service host by a server.
 * @param globular
 * @param application
 * @param domain
 * @param serviceId
 * @param publisherId
 * @param callback
 * @param errorCallback
 */
function getPackageDescriptor(globular, serviceId, publisherId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.GetPackageDescriptorRequest();
    rqst.setServiceid(serviceId);
    rqst.setPublisherid(publisherId);
    globular.resourceService
        .getPackageDescriptor(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResultsList());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.getPackageDescriptor = getPackageDescriptor;
/**
 * Get the list of all service descriptor hosted on a server.
 * @param globular The globular object instance
 * @param application The application name who called the function.
 * @param domain The domain where the application reside.
 * @param callback
 * @param errorCallback
 */
function GetPackagesDescriptor(globular, callback, errorCallback) {
    var rqst = new resource_pb_1.GetPackagesDescriptorRequest();
    var stream = globular.resourceService.getPackagesDescriptor(rqst, {
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var descriptors = new Array();
    stream.on("data", function (rsp) {
        descriptors = descriptors.concat(rsp.getResultsList());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            callback(descriptors);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.GetPackagesDescriptor = GetPackagesDescriptor;
/**
 * Create or update a service descriptor.
 * @param globular
 * @param application
 * @param domain
 * @param descriptor
 * @param callback
 * @param errorCallback
 */
function setServicesDescriptor(globular, descriptor, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new resource_pb_1.SetPackageDescriptorRequest();
    rqst.setPackagedescriptor(descriptor);
    globular.resourceService
        .setPackageDescriptor(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(callback)
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.setServicesDescriptor = setServicesDescriptor;
/**
 * Find services by keywords.
 * @param query
 * @param callback
 */
function FindPackages(globular, keywords, callback, errorCallback) {
    var rqst = new resource_pb_1.FindPackagesDescriptorRequest();
    rqst.setKeywordsList(keywords);
    // Find services by keywords.
    globular.resourceService
        .findPackages(rqst, { application: application.length > 0 ? application : globular.config.IndexApplication, domain: domain, address: address, })
        .then(function (rsp) {
        callback(rsp.getResultsList());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.FindPackages = FindPackages;
/**
 * Return the list of service bundles.
 * @param callback
 */
function getServiceBundles(globular, publisherId, serviceName, serviceId, version, callback, errorCallback) {
    var rqst = new persistence_pb_1.FindRqst();
    rqst.setCollection("ServiceBundle");
    rqst.setDatabase("local_resource");
    rqst.setId("local_resource");
    rqst.setQuery(""); // means all values.
    var stream = globular.persistenceService.find(rqst, {
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address + ":" + window.location.port,
    });
    var data = [];
    stream.on("data", function (rsp) {
        data = mergeTypedArrays(data, rsp.getData());
    });
    stream.on("status", function (status) {
        if (status.code == 0) {
            // filter localy.
            var bundles = JSON.parse(uint8arrayToStringMethod(data));
            callback(bundles.filter(function (bundle) {
                return String(bundle._id).startsWith(publisherId + "%" + serviceName + "%" + version + "%" + serviceId);
            }));
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.getServiceBundles = getServiceBundles;
/**
 * Get the object pointed by a reference.
 * @param globular
 * @param application
 * @param domain
 * @param ref
 * @param callback
 * @param errorCallback
 */
function getReferencedValue(globular, ref, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var database = ref.$db;
    var collection = ref.$ref;
    var rqst = new persistence_pb_1.FindOneRqst();
    rqst.setId(database);
    rqst.setDatabase(database);
    rqst.setCollection(collection);
    rqst.setQuery("{\"_id\":\"".concat(ref.$id, "\"}"));
    rqst.setOptions("");
    globular.persistenceService
        .findOne(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.getReferencedValue = getReferencedValue;
///////////////////////////// Logging Operations ////////////////////////////////////////
/**
 * Read all errors data for server log.
 * @param globular
 * @param application
 * @param domain
 * @param callback
 * @param errorCallback
 */
function readErrors(globular, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var database = "local_resource";
    var collection = "Logs";
    var rqst = new persistence_pb_1.FindOneRqst();
    rqst.setId(database);
    rqst.setDatabase(database);
    rqst.setCollection(collection);
    rqst.setOptions("");
    rqst.setQuery("");
    // call persist data
    var stream = globular.persistenceService.find(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var data;
    // Get the stream and set event on it...
    stream.on("data", function (rsp) {
        data = mergeTypedArrays(data, rsp.getData());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            callback(JSON.parse(uint8arrayToStringMethod(data)));
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.readErrors = readErrors;
/**
 *  Read all logs
 * @param globular
 * @param application
 * @param domain
 * @param query
 * @param callback
 * @param errorCallback
 */
function readLogs(globular, query, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new log_pb_1.GetLogRqst();
    rqst.setQuery(query);
    // call persist data
    var stream = globular.logService.getLog(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var results = new Array();
    // Get the stream and set event on it...
    stream.on("data", function (rsp) {
        results = results.concat(rsp.getInfosList());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            results = results.sort(function (t1, t2) {
                var name1 = t1.getOccurences();
                var name2 = t2.getOccurences();
                if (name1 < name2) {
                    return 1;
                }
                if (name1 > name2) {
                    return -1;
                }
                return 0;
            });
            callback(results);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.readLogs = readLogs;
/**
 * Clear all log of a given type.
 * @param globular
 * @param application
 * @param domain
 * @param logType
 * @param callback
 * @param errorCallback
 */
function clearAllLog(globular, query, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new log_pb_1.ClearAllLogRqst();
    rqst.setQuery(query);
    globular.logService
        .clearAllLog(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(callback)
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.clearAllLog = clearAllLog;
/**
 * Delete log entry.
 * @param globular
 * @param application
 * @param domain
 * @param log
 * @param callback
 * @param errorCallback
 */
function deleteLogEntry(globular, log, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new log_pb_1.DeleteLogRqst();
    rqst.setLog(log);
    globular.logService
        .deleteLog(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(callback)
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.deleteLogEntry = deleteLogEntry;
/**
 * Return the logged method and their count.
 * @param pipeline
 * @param callback
 * @param errorCallback
 */
function getNumbeOfLogsByMethod(globular, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var database = "local_resource";
    var collection = "Logs";
    var rqst = new persistence_pb_1.AggregateRqst();
    rqst.setId(database);
    rqst.setDatabase(database);
    rqst.setCollection(collection);
    rqst.setOptions("");
    var pipeline = "[{\"$group\":{\"_id\":{\"method\":\"$method\"}, \"count\":{\"$sum\":1}}}]";
    rqst.setPipeline(pipeline);
    // call persist data
    var stream = globular.persistenceService.aggregate(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var data = [];
    // Get the stream and set event on it...
    stream.on("data", function (rsp) {
        data = mergeTypedArrays(data, rsp.getData());
    });
    stream.on("status", function (status) {
        if (status.code === 0) {
            callback(JSON.parse(uint8arrayToStringMethod(data)));
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.getNumbeOfLogsByMethod = getNumbeOfLogsByMethod;
///////////////////////////////////// SQL operations /////////////////////////////////
/**
 * Ping globular sql service.
 * @param globular
 * @param application
 * @param domain
 * @param connectionId
 * @param callback
 * @param errorCallback
 */
function pingSql(globular, connectionId, callback, errorCallback, token) {
    if (token === void 0) { token = getToken(); }
    var rqst = new persistence_pb_1.PingConnectionRqst();
    rqst.setId(connectionId);
    globular.sqlService
        .ping(rqst, {
        token: token,
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback(rsp.getResult());
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.pingSql = pingSql;
///////////////////////////// Search Operations ////////////////////////////////////////
/**
 * Search Documents from given database(s) and return results. The search engine in use is
 * xapian, so query must follow xapian query rules.
 * @param globular The server object.
 * @param paths The list of database paths.
 * @param query The query to execute.
 * @param language The language of the database.
 * @param fields The list of field to query, can be empty if all fields must be search or fields are specified in the query.
 * @param offset The offset of resultset
 * @param pageSize The number of result to return
 * @param snippetLength The length of the snippet result.
 * @param callback The success callback
 * @param errorCallback The error callback.
 */
function searchDocuments(globular, paths, query, language, fields, offset, pageSize, snippetLength, callback, errorCallback) {
    var rqst = new search_pb_1.SearchDocumentsRequest();
    rqst.setPathsList(paths);
    rqst.setQuery(query);
    rqst.setLanguage(language);
    rqst.setFieldsList(fields);
    rqst.setOffset(offset);
    rqst.setPagesize(pageSize);
    rqst.setSnippetlength(snippetLength);
    var stream = globular.searchService.searchDocuments(rqst, {
        token: localStorage.getItem("user_token"),
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    var results = new Array();
    // Get the stream and set event on it...
    stream.on("data", function (rsp) {
        results = results.concat(rsp.getResults().getResultsList());
    });
    stream.on("status", function (status) {
        if (status.code == 0) {
            callback(results);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.searchDocuments = searchDocuments;
///////////////////////////////////////////////////////////////////////////////////////////////
// Peers
///////////////////////////////////////////////////////////////////////////////////////////////
function createPeer(globular, name, callback, errorCallback) {
    var rqst = new resource_pb_1.RegisterPeerRqst();
    var peer = new resource_pb_1.Peer();
    peer.setDomain(name);
    rqst.setPeer(peer);
    globular.resourceService
        .registerPeer(rqst, {
        token: localStorage.getItem("user_token"),
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(callback)
        .catch(errorCallback);
}
exports.createPeer = createPeer;
function getAllPeersInfo(globular, callback, errorCallback) {
    var rqst = new resource_pb_1.GetPeersRqst();
    rqst.setQuery("");
    var peers = new Array();
    var stream = globular.resourceService.getPeers(rqst, {
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    });
    // Get the stream and set event on it...
    stream.on("data", function (rsp) {
        peers = peers.concat(rsp.getPeersList());
    });
    stream.on("status", function (status) {
        if (status.code == 0) {
            callback(peers);
        }
        else {
            errorCallback({ message: status.details });
        }
    });
}
exports.getAllPeersInfo = getAllPeersInfo;
function appendActionToPeer(globular, mac, domain, actions, callback, errorCallback) {
    var rqst = new resource_pb_1.AddPeerActionsRqst();
    rqst.setMac(mac);
    rqst.setActionsList(actions);
    globular.resourceService
        .addPeerActions(rqst, {
        token: localStorage.getItem("user_token"),
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.appendActionToPeer = appendActionToPeer;
function removeActionFromPeer(globular, domain, mac, action, callback, errorCallback) {
    var rqst = new resource_pb_1.RemovePeerActionRqst();
    rqst.setMac(mac);
    rqst.setAction(action);
    globular.resourceService
        .removePeerAction(rqst, {
        token: localStorage.getItem("user_token"),
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.removeActionFromPeer = removeActionFromPeer;
function deletePeer(globular, peer, callback, errorCallback) {
    var rqst = new resource_pb_1.DeletePeerRqst();
    rqst.setPeer(peer);
    globular.resourceService
        .deletePeer(rqst, {
        token: localStorage.getItem("user_token"),
        application: application.length > 0 ? application : globular.config.IndexApplication,
        domain: domain, address: address,
    })
        .then(function (rsp) {
        callback();
    })
        .catch(function (err) {
        errorCallback(err);
    });
}
exports.deletePeer = deletePeer;
