import * as grpcWeb from 'grpc-web';

import * as blog_pb from './blog_pb'; // proto import: "blog.proto"


export class BlogServiceClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  createBlogPost(
    request: blog_pb.CreateBlogPostRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: blog_pb.CreateBlogPostResponse) => void
  ): grpcWeb.ClientReadableStream<blog_pb.CreateBlogPostResponse>;

  saveBlogPost(
    request: blog_pb.SaveBlogPostRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: blog_pb.SaveBlogPostResponse) => void
  ): grpcWeb.ClientReadableStream<blog_pb.SaveBlogPostResponse>;

  getBlogPostsByAuthors(
    request: blog_pb.GetBlogPostsByAuthorsRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<blog_pb.GetBlogPostsByAuthorsResponse>;

  getBlogPosts(
    request: blog_pb.GetBlogPostsRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<blog_pb.GetBlogPostsResponse>;

  searchBlogPosts(
    request: blog_pb.SearchBlogPostsRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<blog_pb.SearchBlogPostsResponse>;

  deleteBlogPost(
    request: blog_pb.DeleteBlogPostRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: blog_pb.DeleteBlogPostResponse) => void
  ): grpcWeb.ClientReadableStream<blog_pb.DeleteBlogPostResponse>;

  addEmoji(
    request: blog_pb.AddEmojiRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: blog_pb.AddEmojiResponse) => void
  ): grpcWeb.ClientReadableStream<blog_pb.AddEmojiResponse>;

  removeEmoji(
    request: blog_pb.RemoveEmojiRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: blog_pb.RemoveEmojiResponse) => void
  ): grpcWeb.ClientReadableStream<blog_pb.RemoveEmojiResponse>;

  addComment(
    request: blog_pb.AddCommentRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: blog_pb.AddCommentResponse) => void
  ): grpcWeb.ClientReadableStream<blog_pb.AddCommentResponse>;

  removeComment(
    request: blog_pb.RemoveCommentRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: blog_pb.RemoveCommentResponse) => void
  ): grpcWeb.ClientReadableStream<blog_pb.RemoveCommentResponse>;

}

export class BlogServicePromiseClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  createBlogPost(
    request: blog_pb.CreateBlogPostRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<blog_pb.CreateBlogPostResponse>;

  saveBlogPost(
    request: blog_pb.SaveBlogPostRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<blog_pb.SaveBlogPostResponse>;

  getBlogPostsByAuthors(
    request: blog_pb.GetBlogPostsByAuthorsRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<blog_pb.GetBlogPostsByAuthorsResponse>;

  getBlogPosts(
    request: blog_pb.GetBlogPostsRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<blog_pb.GetBlogPostsResponse>;

  searchBlogPosts(
    request: blog_pb.SearchBlogPostsRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<blog_pb.SearchBlogPostsResponse>;

  deleteBlogPost(
    request: blog_pb.DeleteBlogPostRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<blog_pb.DeleteBlogPostResponse>;

  addEmoji(
    request: blog_pb.AddEmojiRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<blog_pb.AddEmojiResponse>;

  removeEmoji(
    request: blog_pb.RemoveEmojiRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<blog_pb.RemoveEmojiResponse>;

  addComment(
    request: blog_pb.AddCommentRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<blog_pb.AddCommentResponse>;

  removeComment(
    request: blog_pb.RemoveCommentRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<blog_pb.RemoveCommentResponse>;

}

