import * as grpcWeb from 'grpc-web';

import * as conversation_pb from './conversation_pb'; // proto import: "conversation.proto"


export class ConversationServiceClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  stop(
    request: conversation_pb.StopRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.StopResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.StopResponse>;

  connect(
    request: conversation_pb.ConnectRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<conversation_pb.ConnectResponse>;

  disconnect(
    request: conversation_pb.DisconnectRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.DisconnectResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.DisconnectResponse>;

  createConversation(
    request: conversation_pb.CreateConversationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.CreateConversationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.CreateConversationResponse>;

  deleteConversation(
    request: conversation_pb.DeleteConversationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.DeleteConversationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.DeleteConversationResponse>;

  findConversations(
    request: conversation_pb.FindConversationsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.FindConversationsResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.FindConversationsResponse>;

  joinConversation(
    request: conversation_pb.JoinConversationRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<conversation_pb.JoinConversationResponse>;

  leaveConversation(
    request: conversation_pb.LeaveConversationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.LeaveConversationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.LeaveConversationResponse>;

  getConversation(
    request: conversation_pb.GetConversationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.GetConversationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.GetConversationResponse>;

  getConversations(
    request: conversation_pb.GetConversationsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.GetConversationsResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.GetConversationsResponse>;

  kickoutFromConversation(
    request: conversation_pb.KickoutFromConversationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.KickoutFromConversationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.KickoutFromConversationResponse>;

  sendInvitation(
    request: conversation_pb.SendInvitationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.SendInvitationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.SendInvitationResponse>;

  acceptInvitation(
    request: conversation_pb.AcceptInvitationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.AcceptInvitationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.AcceptInvitationResponse>;

  declineInvitation(
    request: conversation_pb.DeclineInvitationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.DeclineInvitationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.DeclineInvitationResponse>;

  revokeInvitation(
    request: conversation_pb.RevokeInvitationRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.RevokeInvitationResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.RevokeInvitationResponse>;

  getReceivedInvitations(
    request: conversation_pb.GetReceivedInvitationsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.GetReceivedInvitationsResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.GetReceivedInvitationsResponse>;

  getSentInvitations(
    request: conversation_pb.GetSentInvitationsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.GetSentInvitationsResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.GetSentInvitationsResponse>;

  sendMessage(
    request: conversation_pb.SendMessageRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.SendMessageResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.SendMessageResponse>;

  deleteMessage(
    request: conversation_pb.DeleteMessageRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.DeleteMessageResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.DeleteMessageResponse>;

  findMessages(
    request: conversation_pb.FindMessagesRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<conversation_pb.FindMessagesResponse>;

  likeMessage(
    request: conversation_pb.LikeMessageRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.LikeMessageResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.LikeMessageResponse>;

  dislikeMessage(
    request: conversation_pb.DislikeMessageRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.DislikeMessageResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.DislikeMessageResponse>;

  setMessageRead(
    request: conversation_pb.SetMessageReadRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: conversation_pb.SetMessageReadResponse) => void
  ): grpcWeb.ClientReadableStream<conversation_pb.SetMessageReadResponse>;

}

export class ConversationServicePromiseClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  stop(
    request: conversation_pb.StopRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.StopResponse>;

  connect(
    request: conversation_pb.ConnectRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<conversation_pb.ConnectResponse>;

  disconnect(
    request: conversation_pb.DisconnectRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.DisconnectResponse>;

  createConversation(
    request: conversation_pb.CreateConversationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.CreateConversationResponse>;

  deleteConversation(
    request: conversation_pb.DeleteConversationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.DeleteConversationResponse>;

  findConversations(
    request: conversation_pb.FindConversationsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.FindConversationsResponse>;

  joinConversation(
    request: conversation_pb.JoinConversationRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<conversation_pb.JoinConversationResponse>;

  leaveConversation(
    request: conversation_pb.LeaveConversationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.LeaveConversationResponse>;

  getConversation(
    request: conversation_pb.GetConversationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.GetConversationResponse>;

  getConversations(
    request: conversation_pb.GetConversationsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.GetConversationsResponse>;

  kickoutFromConversation(
    request: conversation_pb.KickoutFromConversationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.KickoutFromConversationResponse>;

  sendInvitation(
    request: conversation_pb.SendInvitationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.SendInvitationResponse>;

  acceptInvitation(
    request: conversation_pb.AcceptInvitationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.AcceptInvitationResponse>;

  declineInvitation(
    request: conversation_pb.DeclineInvitationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.DeclineInvitationResponse>;

  revokeInvitation(
    request: conversation_pb.RevokeInvitationRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.RevokeInvitationResponse>;

  getReceivedInvitations(
    request: conversation_pb.GetReceivedInvitationsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.GetReceivedInvitationsResponse>;

  getSentInvitations(
    request: conversation_pb.GetSentInvitationsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.GetSentInvitationsResponse>;

  sendMessage(
    request: conversation_pb.SendMessageRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.SendMessageResponse>;

  deleteMessage(
    request: conversation_pb.DeleteMessageRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.DeleteMessageResponse>;

  findMessages(
    request: conversation_pb.FindMessagesRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<conversation_pb.FindMessagesResponse>;

  likeMessage(
    request: conversation_pb.LikeMessageRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.LikeMessageResponse>;

  dislikeMessage(
    request: conversation_pb.DislikeMessageRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.DislikeMessageResponse>;

  setMessageRead(
    request: conversation_pb.SetMessageReadRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<conversation_pb.SetMessageReadResponse>;

}

