import * as grpcWeb from 'grpc-web';

import * as file_pb from './file_pb'; // proto import: "file.proto"


export class FileServiceClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  stop(
    request: file_pb.StopRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.StopResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.StopResponse>;

  addPublicDir(
    request: file_pb.AddPublicDirRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.AddPublicDirResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.AddPublicDirResponse>;

  removePublicDir(
    request: file_pb.RemovePublicDirRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.RemovePublicDirResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.RemovePublicDirResponse>;

  getPublicDirs(
    request: file_pb.GetPublicDirsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.GetPublicDirsResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.GetPublicDirsResponse>;

  readDir(
    request: file_pb.ReadDirRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<file_pb.ReadDirResponse>;

  createDir(
    request: file_pb.CreateDirRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.CreateDirResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.CreateDirResponse>;

  createLnk(
    request: file_pb.CreateLnkRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.CreateLnkResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.CreateLnkResponse>;

  deleteDir(
    request: file_pb.DeleteDirRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.DeleteDirResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.DeleteDirResponse>;

  rename(
    request: file_pb.RenameRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.RenameResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.RenameResponse>;

  move(
    request: file_pb.MoveRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.MoveResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.MoveResponse>;

  copy(
    request: file_pb.CopyRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.CopyResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.CopyResponse>;

  createArchive(
    request: file_pb.CreateArchiveRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.CreateArchiveResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.CreateArchiveResponse>;

  getFileInfo(
    request: file_pb.GetFileInfoRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.GetFileInfoResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.GetFileInfoResponse>;

  getFileMetadata(
    request: file_pb.GetFileMetadataRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.GetFileMetadataResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.GetFileMetadataResponse>;

  readFile(
    request: file_pb.ReadFileRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<file_pb.ReadFileResponse>;

  deleteFile(
    request: file_pb.DeleteFileRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.DeleteFileResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.DeleteFileResponse>;

  getThumbnails(
    request: file_pb.GetThumbnailsRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<file_pb.GetThumbnailsResponse>;

  uploadFile(
    request: file_pb.UploadFileRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<file_pb.UploadFileResponse>;

  writeExcelFile(
    request: file_pb.WriteExcelFileRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.WriteExcelFileResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.WriteExcelFileResponse>;

  htmlToPdf(
    request: file_pb.HtmlToPdfRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: file_pb.HtmlToPdfResponse) => void
  ): grpcWeb.ClientReadableStream<file_pb.HtmlToPdfResponse>;

}

export class FileServicePromiseClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  stop(
    request: file_pb.StopRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.StopResponse>;

  addPublicDir(
    request: file_pb.AddPublicDirRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.AddPublicDirResponse>;

  removePublicDir(
    request: file_pb.RemovePublicDirRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.RemovePublicDirResponse>;

  getPublicDirs(
    request: file_pb.GetPublicDirsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.GetPublicDirsResponse>;

  readDir(
    request: file_pb.ReadDirRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<file_pb.ReadDirResponse>;

  createDir(
    request: file_pb.CreateDirRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.CreateDirResponse>;

  createLnk(
    request: file_pb.CreateLnkRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.CreateLnkResponse>;

  deleteDir(
    request: file_pb.DeleteDirRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.DeleteDirResponse>;

  rename(
    request: file_pb.RenameRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.RenameResponse>;

  move(
    request: file_pb.MoveRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.MoveResponse>;

  copy(
    request: file_pb.CopyRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.CopyResponse>;

  createArchive(
    request: file_pb.CreateArchiveRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.CreateArchiveResponse>;

  getFileInfo(
    request: file_pb.GetFileInfoRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.GetFileInfoResponse>;

  getFileMetadata(
    request: file_pb.GetFileMetadataRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.GetFileMetadataResponse>;

  readFile(
    request: file_pb.ReadFileRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<file_pb.ReadFileResponse>;

  deleteFile(
    request: file_pb.DeleteFileRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.DeleteFileResponse>;

  getThumbnails(
    request: file_pb.GetThumbnailsRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<file_pb.GetThumbnailsResponse>;

  uploadFile(
    request: file_pb.UploadFileRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<file_pb.UploadFileResponse>;

  writeExcelFile(
    request: file_pb.WriteExcelFileRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.WriteExcelFileResponse>;

  htmlToPdf(
    request: file_pb.HtmlToPdfRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<file_pb.HtmlToPdfResponse>;

}

