import * as grpcWeb from 'grpc-web';

import * as ldap_pb from './ldap_pb'; // proto import: "ldap.proto"


export class LdapServiceClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  stop(
    request: ldap_pb.StopRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.StopResponse) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.StopResponse>;

  createConnection(
    request: ldap_pb.CreateConnectionRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.CreateConnectionRsp) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.CreateConnectionRsp>;

  deleteConnection(
    request: ldap_pb.DeleteConnectionRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.DeleteConnectionRsp) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.DeleteConnectionRsp>;

  close(
    request: ldap_pb.CloseRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.CloseRsp) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.CloseRsp>;

  search(
    request: ldap_pb.SearchRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.SearchResp) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.SearchResp>;

  authenticate(
    request: ldap_pb.AuthenticateRqst,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.AuthenticateRsp) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.AuthenticateRsp>;

  synchronize(
    request: ldap_pb.SynchronizeRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.SynchronizeResponse) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.SynchronizeResponse>;

  setLdapSyncInfo(
    request: ldap_pb.setLdapSyncInfoRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.setLdapSyncInfoResponse) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.setLdapSyncInfoResponse>;

  deleteLdapSyncInfo(
    request: ldap_pb.deleteLdapSyncInfoRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.deleteLdapSyncInfoResponse) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.deleteLdapSyncInfoResponse>;

  getLdapSyncInfo(
    request: ldap_pb.getLdapSyncInfoRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: ldap_pb.getLdapSyncInfoResponse) => void
  ): grpcWeb.ClientReadableStream<ldap_pb.getLdapSyncInfoResponse>;

}

export class LdapServicePromiseClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  stop(
    request: ldap_pb.StopRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.StopResponse>;

  createConnection(
    request: ldap_pb.CreateConnectionRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.CreateConnectionRsp>;

  deleteConnection(
    request: ldap_pb.DeleteConnectionRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.DeleteConnectionRsp>;

  close(
    request: ldap_pb.CloseRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.CloseRsp>;

  search(
    request: ldap_pb.SearchRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.SearchResp>;

  authenticate(
    request: ldap_pb.AuthenticateRqst,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.AuthenticateRsp>;

  synchronize(
    request: ldap_pb.SynchronizeRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.SynchronizeResponse>;

  setLdapSyncInfo(
    request: ldap_pb.setLdapSyncInfoRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.setLdapSyncInfoResponse>;

  deleteLdapSyncInfo(
    request: ldap_pb.deleteLdapSyncInfoRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.deleteLdapSyncInfoResponse>;

  getLdapSyncInfo(
    request: ldap_pb.getLdapSyncInfoRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<ldap_pb.getLdapSyncInfoResponse>;

}

