import * as grpcWeb from 'grpc-web';

import * as media_pb from './media_pb'; // proto import: "media.proto"


export class MediaServiceClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  stop(
    request: media_pb.StopRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.StopResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.StopResponse>;

  uploadVideo(
    request: media_pb.UploadVideoRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<media_pb.UploadVideoResponse>;

  createVideoPreview(
    request: media_pb.CreateVideoPreviewRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.CreateVideoPreviewResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.CreateVideoPreviewResponse>;

  createVideoTimeLine(
    request: media_pb.CreateVideoTimeLineRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.CreateVideoTimeLineResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.CreateVideoTimeLineResponse>;

  convertVideoToMpeg4H264(
    request: media_pb.ConvertVideoToMpeg4H264Request,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.ConvertVideoToMpeg4H264Response) => void
  ): grpcWeb.ClientReadableStream<media_pb.ConvertVideoToMpeg4H264Response>;

  convertVideoToHls(
    request: media_pb.ConvertVideoToHlsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.ConvertVideoToHlsResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.ConvertVideoToHlsResponse>;

  startProcessVideo(
    request: media_pb.StartProcessVideoRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.StartProcessVideoResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.StartProcessVideoResponse>;

  startProcessAudio(
    request: media_pb.StartProcessAudioRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.StartProcessAudioResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.StartProcessAudioResponse>;

  stopProcessVideo(
    request: media_pb.StopProcessVideoRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.StopProcessVideoResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.StopProcessVideoResponse>;

  isProcessVideo(
    request: media_pb.IsProcessVideoRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.IsProcessVideoResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.IsProcessVideoResponse>;

  setVideoConversion(
    request: media_pb.SetVideoConversionRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.SetVideoConversionResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.SetVideoConversionResponse>;

  setVideoStreamConversion(
    request: media_pb.SetVideoStreamConversionRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.SetVideoStreamConversionResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.SetVideoStreamConversionResponse>;

  setStartVideoConversionHour(
    request: media_pb.SetStartVideoConversionHourRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.SetStartVideoConversionHourResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.SetStartVideoConversionHourResponse>;

  setMaximumVideoConversionDelay(
    request: media_pb.SetMaximumVideoConversionDelayRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.SetMaximumVideoConversionDelayResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.SetMaximumVideoConversionDelayResponse>;

  getVideoConversionErrors(
    request: media_pb.GetVideoConversionErrorsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.GetVideoConversionErrorsResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.GetVideoConversionErrorsResponse>;

  clearVideoConversionErrors(
    request: media_pb.ClearVideoConversionErrorsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.ClearVideoConversionErrorsResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.ClearVideoConversionErrorsResponse>;

  clearVideoConversionError(
    request: media_pb.ClearVideoConversionErrorRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.ClearVideoConversionErrorResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.ClearVideoConversionErrorResponse>;

  clearVideoConversionLogs(
    request: media_pb.ClearVideoConversionLogsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.ClearVideoConversionLogsResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.ClearVideoConversionLogsResponse>;

  getVideoConversionLogs(
    request: media_pb.GetVideoConversionLogsRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.GetVideoConversionLogsResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.GetVideoConversionLogsResponse>;

  generatePlaylist(
    request: media_pb.GeneratePlaylistRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.GeneratePlaylistResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.GeneratePlaylistResponse>;

  createVttFile(
    request: media_pb.CreateVttFileRequest,
    metadata: grpcWeb.Metadata | undefined,
    callback: (err: grpcWeb.RpcError,
               response: media_pb.CreateVttFileResponse) => void
  ): grpcWeb.ClientReadableStream<media_pb.CreateVttFileResponse>;

}

export class MediaServicePromiseClient {
  constructor (hostname: string,
               credentials?: null | { [index: string]: string; },
               options?: null | { [index: string]: any; });

  stop(
    request: media_pb.StopRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.StopResponse>;

  uploadVideo(
    request: media_pb.UploadVideoRequest,
    metadata?: grpcWeb.Metadata
  ): grpcWeb.ClientReadableStream<media_pb.UploadVideoResponse>;

  createVideoPreview(
    request: media_pb.CreateVideoPreviewRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.CreateVideoPreviewResponse>;

  createVideoTimeLine(
    request: media_pb.CreateVideoTimeLineRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.CreateVideoTimeLineResponse>;

  convertVideoToMpeg4H264(
    request: media_pb.ConvertVideoToMpeg4H264Request,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.ConvertVideoToMpeg4H264Response>;

  convertVideoToHls(
    request: media_pb.ConvertVideoToHlsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.ConvertVideoToHlsResponse>;

  startProcessVideo(
    request: media_pb.StartProcessVideoRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.StartProcessVideoResponse>;

  startProcessAudio(
    request: media_pb.StartProcessAudioRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.StartProcessAudioResponse>;

  stopProcessVideo(
    request: media_pb.StopProcessVideoRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.StopProcessVideoResponse>;

  isProcessVideo(
    request: media_pb.IsProcessVideoRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.IsProcessVideoResponse>;

  setVideoConversion(
    request: media_pb.SetVideoConversionRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.SetVideoConversionResponse>;

  setVideoStreamConversion(
    request: media_pb.SetVideoStreamConversionRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.SetVideoStreamConversionResponse>;

  setStartVideoConversionHour(
    request: media_pb.SetStartVideoConversionHourRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.SetStartVideoConversionHourResponse>;

  setMaximumVideoConversionDelay(
    request: media_pb.SetMaximumVideoConversionDelayRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.SetMaximumVideoConversionDelayResponse>;

  getVideoConversionErrors(
    request: media_pb.GetVideoConversionErrorsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.GetVideoConversionErrorsResponse>;

  clearVideoConversionErrors(
    request: media_pb.ClearVideoConversionErrorsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.ClearVideoConversionErrorsResponse>;

  clearVideoConversionError(
    request: media_pb.ClearVideoConversionErrorRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.ClearVideoConversionErrorResponse>;

  clearVideoConversionLogs(
    request: media_pb.ClearVideoConversionLogsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.ClearVideoConversionLogsResponse>;

  getVideoConversionLogs(
    request: media_pb.GetVideoConversionLogsRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.GetVideoConversionLogsResponse>;

  generatePlaylist(
    request: media_pb.GeneratePlaylistRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.GeneratePlaylistResponse>;

  createVttFile(
    request: media_pb.CreateVttFileRequest,
    metadata?: grpcWeb.Metadata
  ): Promise<media_pb.CreateVttFileResponse>;

}

