"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Globular = exports.EventHub = void 0;
// Here is the list of services from the backend.
var authentication_grpc_web_pb_1 = require("./authentication/authentication_grpc_web_pb");
var event_grpc_web_pb_1 = require("./event/event_grpc_web_pb");
var echo_grpc_web_pb_1 = require("./echo/echo_grpc_web_pb");
var catalog_grpc_web_pb_1 = require("./catalog/catalog_grpc_web_pb");
var file_grpc_web_pb_1 = require("./file/file_grpc_web_pb");
var ldap_grpc_web_pb_1 = require("./ldap/ldap_grpc_web_pb");
var persistence_grpc_web_pb_1 = require("./persistence/persistence_grpc_web_pb");
var mail_grpc_web_pb_1 = require("./mail/mail_grpc_web_pb");
var spc_grpc_web_pb_1 = require("./spc/spc_grpc_web_pb");
var sql_grpc_web_pb_1 = require("./sql/sql_grpc_web_pb");
var storage_grpc_web_pb_1 = require("./storage/storage_grpc_web_pb");
var monitoring_grpc_web_pb_1 = require("./monitoring/monitoring_grpc_web_pb");
var search_grpc_web_pb_1 = require("./search/search_grpc_web_pb");
var admin_grpc_web_pb_1 = require("./admin/admin_grpc_web_pb");
var resource_grpc_web_pb_1 = require("./resource/resource_grpc_web_pb");
var rbac_grpc_web_pb_1 = require("./rbac/rbac_grpc_web_pb");
var log_grpc_web_pb_1 = require("./log/log_grpc_web_pb");
var event_pb_1 = require("./event/event_pb");
var conversation_grpc_web_pb_1 = require("./conversation/conversation_grpc_web_pb");
var blog_grpc_web_pb_1 = require("./blog/blog_grpc_web_pb");
var services_manager_grpc_web_pb_1 = require("./services_manager/services_manager_grpc_web_pb");
var applications_manager_grpc_web_pb_1 = require("./applications_manager/applications_manager_grpc_web_pb");
var discovery_grpc_web_pb_1 = require("./discovery/discovery_grpc_web_pb");
var repository_grpc_web_pb_1 = require("./repository/repository_grpc_web_pb");
var title_grpc_web_pb_1 = require("./title/title_grpc_web_pb");
var torrent_grpc_web_pb_1 = require("./torrent/torrent_grpc_web_pb");
var media_grpc_web_pb_1 = require("./media/media_grpc_web_pb");
/**
 * Create a "version 4" RFC-4122 UUID (Universal Unique Identifier) string.
 * @returns {string} A string containing the UUID.
 */
function randomUUID() {
    var s = new Array();
    var itoh = '0123456789abcdef'; // Make array of random hex digits. The UUID only has 32 digits in it, but we
    // allocate an extra items to make room for the '-'s we'll be inserting.
    for (var i = 0; i < 36; i++)
        s[i] = Math.floor(Math.random() * 0x10); // Conform to RFC-4122, section 4.4
    s[14] = 4; // Set 4 high bits of time_high field to version
    s[19] = s[19] & 0x3 | 0x8; // Specify 2 high bits of clock sequence
    // Convert to hex chars
    for (var i = 0; i < 36; i++)
        s[i] = itoh[s[i]]; // Insert '-'s
    s[8] = s[13] = s[18] = s[23] = '-';
    return s.join('');
}
window.onbeforeunload = function (e) {
    console.log("cleanup network resources...");
    return undefined;
};
/**
 * That local and distant event hub.
 */
var EventHub = /** @class */ (function () {
    /**
     * @param {*} service If undefined only local event will be allow.
     */
    function EventHub(g) {
        // The parent globular object.
        this.globular = g;
        // Subscriber function map.
        this.subscribers = {};
        // Keep references of object that call subscribe...
        this.refs = {};
        // This is the client uuid
        this.uuid = randomUUID();
        // This will stay there until disconnect will be call...
        this.connect(null);
        // Disconnect null object
        this.removeDeletedListeners();
    }
    // Disconnect null listeners...
    EventHub.prototype.removeDeletedListeners = function () {
        var _this = this;
        setInterval(function () {
            for (var uuid in _this.refs) {
                if (_this.refs[uuid] == null) {
                    _this.unSubscribe(uuid.split(":")[0], uuid.split(":")[1]);
                }
            }
        }, 5000);
    };
    EventHub.prototype.reinitRemoteListeners = function () {
        var _this = this;
        var subscriptions = [];
        for (var name_1 in this.subscribers) {
            for (var uuid in this.subscribers[name_1]) {
                var subscription = this.subscribers[name_1][uuid];
                if (!subscription.local) {
                    if (subscriptions.indexOf(name_1) == -1) {
                        subscriptions.push(name_1);
                    }
                }
            }
        }
        var subscribe = function () {
            var name = subscriptions.pop();
            var rqst = new event_pb_1.SubscribeRequest;
            rqst.setName(name);
            rqst.setUuid(_this.uuid);
            _this.globular.eventService.subscribe(rqst).then(function (rsp) {
                if (subscriptions.length > 0) {
                    subscribe();
                }
            }).catch(function (err) { console.log(err); });
        };
        subscribe();
    };
    /** Connect to the remote server. */
    EventHub.prototype.connect = function (callback) {
        var _this = this;
        // Open the connection with the server.
        if (this.globular.eventService !== undefined) {
            // The first step is to subscribe to an event channel.
            var rqst = new event_pb_1.OnEventRequest();
            rqst.setUuid(this.uuid);
            var stream_1 = this.globular.eventService.onEvent(rqst, {});
            var last_timeout_1 = -1;
            // Get the stream and set event on it...
            stream_1.on('data', function (rsp) {
                if (rsp.hasEvt()) {
                    var evt = rsp.getEvt();
                    var bytes = evt.getData();
                    var data = "";
                    if (bytes != null) {
                        if (bytes.length > 0) {
                            data = new TextDecoder("utf-8").decode(bytes);
                        }
                    }
                    // dispatch the event localy.
                    _this.dispatch(evt.getName(), data);
                }
                else if (rsp.hasKa()) {
                    /**  */
                    if (last_timeout_1 > 0) {
                        clearTimeout(last_timeout_1);
                    }
                    // wait for the next timeout...
                    last_timeout_1 = setTimeout(function () {
                        stream_1.cancel(); // cancel the stream if it was active
                        _this.globular.resetEventService(); // reset listeners...
                        _this.connect(function () {
                            _this.reinitRemoteListeners();
                        });
                    }, 25 * 1000); // if signal was received for 25 second I will try to reconnect...
                }
            });
            stream_1.on('status', function (status) {
                if (status.code != 0) {
                    /** nothing here. */
                }
            });
            stream_1.on('end', function () {
                /** Nothing here */
            });
            if (callback != undefined) {
                callback();
            }
        } /* else {
          // Wait a second before try to connect agin...
          setTimeout(() => {
            this.connect(() => {
              this.reinitRemoteListeners()
            })
    
          }, 1000)
        }*/
    };
    /**
     *
     * @param {*} name The name of the event to subcribe to.
     * @param {*} onsubscribe That function return the uuid of the subscriber.
     * @param {*} onevent That function is call when the event is use.
     * @param local if true means the event run localy, true by default
     * @param ref The reference to the object that subscribe to this event...
     */
    EventHub.prototype.subscribe = function (name, onsubscribe, onevent, local, ref) {
        var _this = this;
        if (local === void 0) { local = true; }
        if (ref === void 0) { ref = null; }
        // Register the local subscriber.
        var uuid = randomUUID();
        if (ref != null) {
            if (ref[name] != null) {
                // A subscription already exist...
                return;
            }
            this.refs[uuid + ":" + name] = ref;
            // keep the listner uuid...
            ref[name] = uuid;
        }
        if (!local) {
            if (this.subscribers[name] == undefined) {
                this.subscribers[name] = {};
                var rqst = new event_pb_1.SubscribeRequest;
                rqst.setName(name);
                rqst.setUuid(this.uuid);
                this.globular.eventService.subscribe(rqst, {}).then(function (rsp) {
                    if (_this.subscribers[name] == undefined) {
                        _this.subscribers[name] = {}; // create if it not exist.
                    }
                    _this.subscribers[name][uuid] = { onsubscribe: onsubscribe, onevent: onevent, local: local };
                    onsubscribe(uuid);
                });
            }
            else {
                this.subscribers[name][uuid] = { onsubscribe: onsubscribe, onevent: onevent, local: local };
                onsubscribe(uuid);
            }
        }
        else {
            // create a uuid and call onsubscribe callback.
            if (this.subscribers[name] === undefined) {
                this.subscribers[name] = {};
            }
            this.subscribers[name][uuid] = { onsubscribe: onsubscribe, onevent: onevent, local: local };
            onsubscribe(uuid);
        }
    };
    /**
     *
     * @param {*} name
     * @param {*} uuid
     */
    EventHub.prototype.unSubscribe = function (name, uuid) {
        if (this.subscribers[name] === undefined) {
            return;
        }
        if (this.subscribers[name][uuid] === undefined) {
            return;
        }
        // remove references...
        var ref = this.refs[name + ":" + uuid];
        if (ref != null) {
            delete ref[name];
            delete this.refs[name + ":" + uuid];
        }
        var subscription = this.subscribers[name][uuid];
        // Remove the local subscriber.
        delete this.subscribers[name][uuid];
        if (Object.keys(this.subscribers[name]).length === 0) {
            delete this.subscribers[name];
            // disconnect from the distant server.
            if (this.globular.eventService !== undefined && !subscription.local) {
                var rqst = new event_pb_1.UnSubscribeRequest();
                rqst.setName(name);
                rqst.setUuid(this.uuid);
                // Now I will test with promise
                this.globular.eventService.unSubscribe(rqst, {})
                    .then(function (resp) {
                })
                    .catch(function (error) {
                    console.log(error);
                });
            }
        }
    };
    /**
     * Publish an event on the bus, or locally in case of local event.
     * @param {*} name The  name of the event to publish
     * @param {*} data The data associated with the event
     * @param {*} local If the event is not local the data must be seraliaze before sent.
     */
    EventHub.prototype.publish = function (name, data, local) {
        if (local === true) {
            this.dispatch(name, data);
        }
        else {
            // Create a new request.
            var rqst = new event_pb_1.PublishRequest();
            var evt = new event_pb_1.Event();
            evt.setName(name);
            var enc = new TextEncoder(); // always utf-8
            // encode the string to a array of byte
            if (data != null) {
                if (data.length > 0) {
                    evt.setData(enc.encode(data));
                }
            }
            rqst.setEvt(evt);
            // Now I will test with promise
            this.globular.eventService.publish(rqst, {})
                .then(function (resp) {
                /** Nothing to do here. */
            })
                .catch(function (error) {
                console.log(error);
            });
        }
    };
    /** Dispatch the event localy */
    EventHub.prototype.dispatch = function (name, data) {
        for (var uuid in this.subscribers[name]) {
            // call the event callback function.
            if (this.subscribers !== undefined) {
                if (this.subscribers[name] !== undefined) {
                    if (this.subscribers[name][uuid] !== undefined) {
                        this.subscribers[name][uuid].onevent(data);
                    }
                }
            }
        }
    };
    return EventHub;
}());
exports.EventHub = EventHub;
// Get the configuration from url
function getFileConfig(url, callback, errorcallback) {
    // so I will try to get the configuration from the server...
    var xmlhttp = new XMLHttpRequest();
    xmlhttp.timeout = 1500;
    xmlhttp.onreadystatechange = function () {
        if (this.readyState == 4 && this.status == 201) {
            var obj = JSON.parse(this.responseText);
            callback(obj);
        }
        else if (this.readyState == 4) {
            // Create a URL object
            var url_ = new URL(url);
            var port = url_.port;
            if (port == "") {
                if (url_.protocol == "https:") {
                    port = "443";
                }
                else {
                    port = "80";
                }
            }
            // so I will try to get the configuration from the server...
            var xmlhttp = new XMLHttpRequest();
            xmlhttp.timeout = 1500;
            xmlhttp.onreadystatechange = function () {
                if (this.readyState == 4 && this.status == 201) {
                    var obj = JSON.parse(this.responseText);
                    callback(obj);
                }
                else if (this.readyState == 4) {
                    // will try to get the config at host level...
                    // Extract the hostname (domain) from the URL
                    var hostname = url_.hostname;
                    // Split the hostname by dots
                    var parts = hostname.split('.');
                    // Get the last two parts to form the top-level domain
                    var topLevelDomain = parts.slice(-2).join('.');
                    var xmlhttp = new XMLHttpRequest();
                    xmlhttp.timeout = 1500;
                    xmlhttp.onreadystatechange = function () {
                        if (this.readyState == 4 && this.status == 201) {
                            var obj = JSON.parse(this.responseText);
                            callback(obj);
                        }
                        else if (this.readyState == 4) {
                            var xmlhttp = new XMLHttpRequest();
                            xmlhttp.timeout = 1500;
                            xmlhttp.onreadystatechange = function () {
                                if (this.readyState == 4 && this.status == 201) {
                                    var obj = JSON.parse(this.responseText);
                                    callback(obj);
                                }
                                else if (this.readyState == 4) {
                                    errorcallback("fail to get the configuration file at url " + url + " status " + this.status);
                                }
                            };
                            // try to get config from the actual server with the host and port.
                            xmlhttp.open("GET", "http://" + topLevelDomain + "/config?host=".concat(url_.hostname, "&port=").concat("80"), true);
                            xmlhttp.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
                            xmlhttp.send();
                        }
                    };
                    // try to get config from the actual server with the host and port.
                    xmlhttp.open("GET", url_.protocol + "//" + topLevelDomain + "/config?host=".concat(url_.hostname, "&port=").concat(port), true);
                    xmlhttp.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
                    xmlhttp.send();
                }
            };
            // try to get config from the actual server with the host and port.
            xmlhttp.open("GET", "/config?host=".concat(url_.hostname, "&port=").concat(port), true);
            xmlhttp.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            xmlhttp.send();
        }
    };
    // Create a URL object
    var url_ = new URL(url);
    var port = url_.port;
    if (port == "") {
        if (url_.protocol == "https:") {
            port = "443";
        }
        else {
            port = "80";
        }
    }
    // try to get the url directly...
    xmlhttp.open("GET", url, true);
    xmlhttp.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
    xmlhttp.send();
}
/**
 * Globular regroup all serivces in one object that can be use by
 * application to get access to sql, ldap, persistence... service.
 */
var Globular = /** @class */ (function () {
    /** The configuation. */
    function Globular(url, callback, errorcallback) {
        var _this = this;
        // Keep the config...
        getFileConfig(url, function (config) {
            _this.config = config;
            // force event hub initialysation...
            _this._services = {};
            // here I will connect the event hub...
            _this._eventHub = new EventHub(_this);
            // I will subscribe on services configuration update.
            _this._eventHub.subscribe("update_globular_service_configuration_evt", function () { }, function (evt) {
                var config = JSON.parse(evt);
                _this._services[config.Id] = null;
                _this.config.Services[config.Id] = config;
                // console.log("service has change !", config)
            }, false, _this);
            callback();
        }, errorcallback);
    }
    Object.defineProperty(Globular.prototype, "eventHub", {
        get: function () {
            return this._eventHub;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "config", {
        get: function () {
            return this._config;
        },
        set: function (value) {
            this._config = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "address", {
        // Return the domain of a globule from it configuration.
        get: function () {
            var domain = this._config.Name;
            // if the domain is localhost i will use the hostname as address.
            if (this._config.Domain != "localhost") {
                if (domain.length > 0 && !this._config.Domain.startsWith(this._config.Name)) {
                    if (this._config.Domain.length > 0) {
                        domain += "." + this._config.Domain;
                    }
                }
                else {
                    domain = this._config.Domain;
                }
            }
            return domain;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "domain", {
        // Return the domain of a globule from it configuration.
        get: function () {
            return this._config.Domain;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "adminService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._adminService == null) {
                var configs = this.getConfigs('admin.AdminService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._adminService = new admin_grpc_web_pb_1.AdminServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._adminService;
                });
            }
            return this._adminService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "servicesManagerService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._servicesManagerService == null) {
                var configs = this.getConfigs('services_manager.ServicesManagerService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._servicesManagerService = new services_manager_grpc_web_pb_1.ServicesManagerServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._servicesManagerService;
                });
            }
            return this._servicesManagerService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "applicationsManagerService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._applicationsManagerService == null) {
                var configs = this.getConfigs('applications_manager.ApplicationManagerService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._applicationsManagerService = new applications_manager_grpc_web_pb_1.ApplicationManagerServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._applicationsManagerService;
                });
            }
            return this._applicationsManagerService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "authenticationService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._authenticationService == null) {
                var configs = this.getConfigs('authentication.AuthenticationService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._authenticationService = new authentication_grpc_web_pb_1.AuthenticationServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._authenticationService;
                });
            }
            return this._authenticationService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "resourceService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._resourceService == null) {
                var configs = this.getConfigs('resource.ResourceService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._resourceService = new resource_grpc_web_pb_1.ResourceServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._resourceService;
                });
            }
            return this._resourceService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "logService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._logService == null) {
                var configs = this.getConfigs('log.LogService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._logService = new log_grpc_web_pb_1.LogServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._logService;
                });
            }
            return this._logService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "mediaService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._mediaService == null) {
                var configs = this.getConfigs('media.MediaService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._mediaService = new media_grpc_web_pb_1.MediaServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._mediaService;
                });
            }
            return this._mediaService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "blogService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._blogService == null) {
                var configs = this.getConfigs('blog.BlogService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._blogService = new blog_grpc_web_pb_1.BlogServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._blogService;
                });
            }
            return this._blogService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "conversationService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._conversationService == null) {
                var configs = this.getConfigs('conversation.ConversationService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._conversationService = new conversation_grpc_web_pb_1.ConversationServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._conversationService;
                });
            }
            return this._conversationService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "rbacService", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._rbacService == null) {
                var configs = this.getConfigs('rbac.RbacService');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._rbacService = new rbac_grpc_web_pb_1.RbacServicePromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._rbacService;
                });
            }
            return this._rbacService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "packagesDicovery", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._packagesDicovery == null) {
                var configs = this.getConfigs('discovery.PackageDiscovery');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._packagesDicovery = new discovery_grpc_web_pb_1.PackageDiscoveryPromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._packagesDicovery;
                });
            }
            return this._packagesDicovery;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "servicesRepository", {
        get: function () {
            var _this = this;
            // refresh the config.
            if (this._servicesRepository == null) {
                var configs = this.getConfigs('repository.PackageRepository');
                configs.forEach(function (config) {
                    var serviceAddress = config.Address.split(":")[0];
                    if (serviceAddress == "localhost") {
                        serviceAddress = _this.address;
                    }
                    _this._servicesRepository = new repository_grpc_web_pb_1.PackageRepositoryPromiseClient(_this.config.Protocol +
                        '://' +
                        serviceAddress +
                        ':' +
                        config.Proxy, null, null);
                    _this._services[config.Id] = _this._servicesRepository;
                });
            }
            return this._servicesRepository;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "catalogService", {
        get: function () {
            var _this = this;
            if (this._catalogService == null) {
                var configs = this.getConfigs('catalog.CatalogService');
                configs.forEach(function (config) {
                    if (_this._catalogService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._catalogService = new catalog_grpc_web_pb_1.CatalogServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._catalogService;
                    }
                });
            }
            return this._catalogService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "echoService", {
        get: function () {
            var _this = this;
            if (this._echoService == null) {
                var configs = this.getConfigs('echo.EchoService');
                configs.forEach(function (config) {
                    if (_this._echoService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._echoService = new echo_grpc_web_pb_1.EchoServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._echoService;
                    }
                });
            }
            return this._echoService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "eventService", {
        get: function () {
            var _this = this;
            if (this._eventService == null) {
                var configs = this.getConfigs('event.EventService');
                configs.forEach(function (config) {
                    if (_this._eventService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._eventService = new event_grpc_web_pb_1.EventServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._eventService;
                    }
                });
            }
            return this._eventService;
        },
        enumerable: false,
        configurable: true
    });
    Globular.prototype.resetEventService = function () {
        this._eventService = undefined;
    };
    Object.defineProperty(Globular.prototype, "fileService", {
        get: function () {
            var _this = this;
            if (this._fileService == null) {
                var configs = this.getConfigs('file.FileService');
                configs.forEach(function (config) {
                    if (_this._fileService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._fileService = new file_grpc_web_pb_1.FileServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._fileService;
                    }
                });
            }
            return this._fileService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "ldapService", {
        get: function () {
            var _this = this;
            if (this._ldapService == null) {
                var configs = this.getConfigs('ldap.LdapService');
                configs.forEach(function (config) {
                    if (_this._ldapService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._ldapService = new ldap_grpc_web_pb_1.LdapServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._ldapService;
                    }
                });
            }
            return this._ldapService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "persistenceService", {
        get: function () {
            var _this = this;
            if (this._persistenceService == null) {
                var configs = this.getConfigs('persistence.PersistenceService');
                configs.forEach(function (config) {
                    if (_this._persistenceService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._persistenceService = new persistence_grpc_web_pb_1.PersistenceServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._persistenceService;
                    }
                });
            }
            return this._persistenceService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "mailService", {
        get: function () {
            var _this = this;
            if (this._mailService == null) {
                var configs = this.getConfigs('mail.MailService');
                configs.forEach(function (config) {
                    if (_this._mailService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._mailService = new mail_grpc_web_pb_1.MailServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._mailService;
                    }
                });
            }
            return this._mailService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "sqlService", {
        get: function () {
            var _this = this;
            if (this._sqlService == null) {
                var configs = this.getConfigs('sql.SqlService');
                configs.forEach(function (config) {
                    if (_this._sqlService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._sqlService = new sql_grpc_web_pb_1.SqlServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._sqlService;
                    }
                });
            }
            return this._sqlService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "storageService", {
        get: function () {
            var _this = this;
            if (this._storageService == null) {
                var configs = this.getConfigs('storage.StorageService');
                configs.forEach(function (config) {
                    if (_this._storageService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._storageService = new storage_grpc_web_pb_1.StorageServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._storageService;
                    }
                });
            }
            return this._storageService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "monitoringService", {
        get: function () {
            var _this = this;
            if (this._monitoringService == null) {
                var configs = this.getConfigs('monitoring.MonitoringService');
                configs.forEach(function (config) {
                    if (_this._monitoringService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._monitoringService = new monitoring_grpc_web_pb_1.MonitoringServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._monitoringService;
                    }
                });
            }
            return this._monitoringService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "spcService", {
        get: function () {
            var _this = this;
            if (this._spcService == null) {
                var configs = this.getConfigs('spc.SpcService');
                configs.forEach(function (config) {
                    if (_this._spcService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._spcService = new spc_grpc_web_pb_1.SpcServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._spcService;
                    }
                });
            }
            return this._spcService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "titleService", {
        get: function () {
            var _this = this;
            if (this._titleService == null) {
                var configs = this.getConfigs('title.TitleService');
                configs.forEach(function (config) {
                    if (_this._titleService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._titleService = new title_grpc_web_pb_1.TitleServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._titleService;
                    }
                });
            }
            return this._titleService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "torrentService", {
        get: function () {
            var _this = this;
            if (this._torrentService == null) {
                var configs = this.getConfigs('torrent.TorrentService');
                configs.forEach(function (config) {
                    if (_this._torrentService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._torrentService = new torrent_grpc_web_pb_1.TorrentServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._torrentService;
                    }
                });
            }
            return this._torrentService;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Globular.prototype, "searchService", {
        get: function () {
            var _this = this;
            if (this._searchService == null) {
                var configs = this.getConfigs('search.SearchService');
                configs.forEach(function (config) {
                    if (_this._searchService == null) {
                        var serviceAddress = config.Address.split(":")[0];
                        if (serviceAddress == "localhost") {
                            serviceAddress = _this.address;
                        }
                        _this._searchService = new search_grpc_web_pb_1.SearchServicePromiseClient(_this.config.Protocol +
                            '://' +
                            serviceAddress +
                            ':' +
                            config.Proxy, null, null);
                        _this._services[config.Id] = _this._searchService;
                    }
                });
            }
            return this._searchService;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Return a service instance
     * @param name The name of the service.
     * @param id The id of the service.
     *
     * as example to get a specific instance of the file services.
     *  globular.getServiceById("file.fileService", "applications")
     */
    Globular.prototype.getServiceById = function (name, id) {
        for (var id_ in this.config.Services) {
            var service = this.config.Services[id];
            if (service.Name == name && id_ == id) {
                return service;
            }
        }
    };
    // Return the first configuration that match the given name.
    // The load balancer will be in charge to select the correct service instance from the list
    // The first instance is the entry point of the services.
    Globular.prototype.getConfigs = function (name) {
        var services = new Array();
        for (var id in this.config.Services) {
            var service = this.config.Services[id];
            if (service.Name == name) {
                services.push(service);
            }
        }
        return services;
    };
    return Globular;
}());
exports.Globular = Globular;
